#include <QtGui>
#include <iostream>
#include <fstream>
#include "bsyorkitem.h"
#include "pt2i.h"

using namespace std;


const int BSYorkItem::YORK_ALONE = 0;
const int BSYorkItem::CANNY_ALONE = 1;
const int BSYorkItem::DILATED_CANNY_ALONE = 2;
const int BSYorkItem::YORK_OVER_CANNY = 3;
const int BSYorkItem::YORK_OVER_BS = 4;
const int BSYorkItem::YORK_OVER_DSS = 5;
const int BSYorkItem::YORK_WITH_CANNY = 6;
const int BSYorkItem::YORK_WITH_DSS = 7;
const int BSYorkItem::NO_INFO = 8;
const QColor BSYorkItem::OVER_COLOR = Qt::lightGray;
const QColor BSYorkItem::UNDER_COLOR = Qt::yellow;
const QColor BSYorkItem::BOTH_COLOR = Qt::black;
const int BSYorkItem::DEFAULT_PEN_WIDTH = 1;


BSYorkItem::BSYorkItem (int width, int height, const QImage *im,
                          BSDetector *detector)
{
  w = width;
  h = height;
  int size = w * h;
  this->im = im;
  bsmap = new bool[size];
  clmap = new bool[size];
  det = detector;
  type = YORK_ALONE;
  cannyDilation = 0;
  yorks = new ExtLines ("yorklines.txt", w, h);
  cannys = new ExtLines ("cannylines.txt", w, h, 5);
}


BSYorkItem::~BSYorkItem ()
{
  delete [] bsmap;
}


QRectF BSYorkItem::boundingRect () const
{
  return QRectF (0, 0, w - 1, h - 1); // ZZZ
}


void BSYorkItem::paint (QPainter *painter,
                         const QStyleOptionGraphicsItem *option,
                         QWidget *widget)
{
  Q_UNUSED (option);
  Q_UNUSED (widget);

  for (int i = 0; i < w * h; i++) bsmap[i] = false;
  if (type == YORK_ALONE) drawYorkLines (painter);
  else if (type == CANNY_ALONE) drawCannyLines (painter);
  else if (type == DILATED_CANNY_ALONE) drawDilatedCannyLines (painter);
  else if (type == YORK_OVER_CANNY)
  {
    drawCannyLines (painter);
    drawYorkLines (painter);
  }
  else if (type == YORK_OVER_BS)
  {
    drawBS (painter);
    drawYorkLines (painter);
  }
  else if (type == YORK_OVER_DSS)
  {
    drawDSS (painter);
    drawYorkLines (painter);
  }
  else if (type == YORK_WITH_CANNY)
  {
    drawCannyLines (painter, false);
    drawYorkLines (painter);
  }
  else if (type == YORK_WITH_DSS)
  {
    drawDSS (painter, false);
    drawYorkLines (painter);
  }
  else drawYorkLines (painter);
  int nboth = 0, ncl = 0, nbs = 0;
  for (int i = 0; i < w * h; i++)
    if (clmap[i])
      if (bsmap[i]) nboth++;
      else ncl++;
    else if (bsmap[i]) nbs++;
  cout << nboth << " both, " << ncl << " york, " << nbs << " bs" << endl;
}


void BSYorkItem::drawBS (QPainter *painter)
{
  painter->setPen (QPen (UNDER_COLOR, DEFAULT_PEN_WIDTH, Qt::SolidLine,
                         Qt::RoundCap, Qt::RoundJoin));
  vector<BlurredSegment *> bss = det->getBlurredSegments ();
  if (! bss.empty ())
  {
    vector<BlurredSegment *>::const_iterator it = bss.begin ();
    while (it != bss.end ())
    {
      if (*it != NULL) drawPoints (painter, (*it)->getAllPoints ());
      it++;
    }
  }
}


void BSYorkItem::drawDSS (QPainter *painter, bool disp)
{
  painter->setPen (QPen (UNDER_COLOR, DEFAULT_PEN_WIDTH, Qt::SolidLine,
                         Qt::RoundCap, Qt::RoundJoin));
  vector<BlurredSegment *> bss = det->getBlurredSegments ();
  if (! bss.empty ())
  {
    vector<BlurredSegment *>::const_iterator it = bss.begin ();
    while (it != bss.end ())
    {
      if ((*it) != NULL)
      {
        DigitalStraightSegment *dss = (*it)->getSegment ();
        if (dss != NULL)
        {
          vector<Pt2i> pts;
          dss->getPoints (pts);
          drawPoints (painter, pts, disp);
        }  
      }
      it++;
    }
  }
}


void BSYorkItem::drawPoints (QPainter *painter, vector<Pt2i> pts, bool disp)
{
  vector<Pt2i>::iterator iter = pts.begin ();
  while (iter != pts.end ())
  {
    Pt2i p = *iter++;
    if (p.x() < w && p.y() < h && p.x() >= 0 && p.y() >= 0)
    {
      bsmap[p.y()*w+p.x()] = true;
      if (disp) painter->drawPoint (QPoint (p.x(), h - 1 - p.y()));  // dec 1
    }
  }
}


void BSYorkItem::drawYorkLines (QPainter *painter)
{
  int n = 0;
  Pt2i *pts = NULL;
  int nb = yorks->countOfLines ();
  for (int i = 0; i < nb; i++)
  {
    pts = Pt2i (yorks->xStart (i), yorks->yStart (i)).drawing (
          Pt2i (yorks->xEnd (i), yorks->yEnd (i)), &n);
    for (int i = 0; i < n; i++)
    {
      if (pts[i].x () >= 0 && pts[i].x () < w
          && pts[i].y () >= 0 && pts[i].y () < h)
      {
        painter->setPen (QPen (bsmap[pts[i].y()*w+pts[i].x()] ?
                               BOTH_COLOR : OVER_COLOR,
                           DEFAULT_PEN_WIDTH, Qt::SolidLine,
                           Qt::RoundCap, Qt::RoundJoin));
        painter->drawPoint (QPoint (pts[i].x (),
                                    h - 1 - pts[i].y ()));  // dec 1
        clmap[pts[i].y()*w+pts[i].x()] = true;
      }
    }
    delete [] pts;
  }
}


void BSYorkItem::drawCannyLines (QPainter *painter, bool disp) {
  int n = 0;
  Pt2i *pts = NULL;
  int nb = cannys->countOfLines ();
  for (int i = 0; i < nb; i++)
  {
    pts = Pt2i (cannys->xStart (i), cannys->yStart (i)).drawing (
          Pt2i (cannys->xEnd (i), cannys->yEnd (i)), &n);
    for (int i = 0; i < n; i++)
    {
      if (pts[i].x () >= 0 && pts[i].x () < w
          && pts[i].y () >= 0 && pts[i].y () < h)
      {
        if (disp)
        {
          painter->setPen (QPen (UNDER_COLOR,
                             DEFAULT_PEN_WIDTH, Qt::SolidLine,
                             Qt::RoundCap, Qt::RoundJoin));
          painter->drawPoint (QPoint (pts[i].x (),
                                      h - 1 - pts[i].y ()));  // dec 1
        }
        bsmap[pts[i].y()*w+pts[i].x()] = true;
      }
    }
    delete [] pts;
  }
}


void BSYorkItem::drawDilatedCannyLines (QPainter *painter, bool disp)
{
  int nb = cannys->countOfLines ();
  for (int i = 0; i < nb; i++)
  {
    int xs = cannys->xStart (i);
    int ys = cannys->yStart (i);
    int xe = cannys->xEnd (i);
    int ye = cannys->yEnd (i);
    int minx = (xe < xs ? xe : xs);
    int miny = (ye < ys ? ye : ys);
    int maxx = (xe < xs ? xs : xe);
    int maxy = (ye < ys ? ys : ye);
    DigitalStraightSegment *dss = new DigitalStraightSegment (
      Pt2i (xs, ys), Pt2i (xe, ye),
      DigitalStraightLine::DSL_NAIVE, minx, miny, maxx, maxy);
    vector<Pt2i> points;
    if (cannyDilation > 0)
    {
      int dil = 0;
      if (cannyDilation == 1) dil = dss->period ();
      else if (cannyDilation == 2) dil = dss->standard ();
      else if (cannyDilation == 3) dil = 2 * dss->period ();
cout << "Dilation of " << dil << endl;
      dss->dilate (dil);
    }
    dss->getPoints (points);
    vector<Pt2i>::iterator pit = points.begin ();
    while (pit != points.end ())
    {
      if (pit->x () >= 0 && pit->x () < w && pit->y () >= 0 && pit->y () < h)
      {
        if (disp)
        {
          painter->setPen (QPen (UNDER_COLOR,
                             DEFAULT_PEN_WIDTH, Qt::SolidLine,
                             Qt::RoundCap, Qt::RoundJoin));
          painter->drawPoint (QPoint (pit->x (), h - 1 - pit->y ()));
        }
        bsmap[pit->y()*w+pit->x()] = true;
      }
      pit ++;
    }
    delete dss;
  }
}


void BSYorkItem::computeBScovering ()
{
  det->detectAll ();
  double fbsdCov = yorks->covering (det->getBlurredSegments ());
  cout << "FBSD covering : " << fbsdCov << endl;
  int nb;
  double lg;
  double cannyCov = yorks->cannyLineCovering (
                      "/home/even/tmp/FBSD/cannylines.txt",
                      nb, lg, det->finalSpreadMinLength ());
  cout << "Canny lines covering : " << cannyCov << endl;

}


bool BSYorkItem::toggleCannyDilation ()
{
  if (++cannyDilation == 4) cannyDilation = 0;
  return (type == DILATED_CANNY_ALONE);
}
