#ifndef BS_YORK_ITEM_H
#define BS_YORK_ITEM_H

#include <QGraphicsItem>
#include "bsdetector.h"
#include "extlines.h"

using namespace std;


/** 
 * @class BSYorkItem bsyorkitem.h
 * \brief Blurred segment comparator with YorkLine.
 * \author {P. Even}
 */
class BSYorkItem : public QGraphicsItem
{
public:

  /**
   * \brief Creates a YorkLine comparator.
   */
  BSYorkItem (int width, int height, const QImage *im, BSDetector *detector);

  /**
   * \brief Deletes the YorkLine comparator.
   */
  ~BSYorkItem ();

  /**
   * \brief Returns the grid area.
   */
  QRectF boundingRect () const;

  /**
   * \brief Redraws the YorkLine comparator grid.
   */
  void paint (QPainter *painter,
              const QStyleOptionGraphicsItem *option, QWidget *widget);

  /**
   * \brief Toggles display modality.
   */
  inline void toggleType (bool back) {
    if (back) { if (--type < 0) type = NO_INFO - 1; }
    else if (++type == NO_INFO) type = 0; }

  /**
   * \brief Toggles canny dilation.
   * Returns if the display should be updated.
   */
  bool toggleCannyDilation ();

  /**
   * \brief Cpomputes and displays blurred segment covering.
   */
  void computeBScovering ();

  /**
   * \brief Returns the displayed information title.
   */
  inline QString itemTitle () const {
    if (type == YORK_ALONE)
      return ("York Urban DB ground truth");
    else if (type == DILATED_CANNY_ALONE)
      return ("Dilated_Canny lines");
    else if (type == CANNY_ALONE)
      return ("Canny lines");
    else if (type == YORK_OVER_CANNY)
      return ("York over Canny lines");
    else if (type == YORK_OVER_BS)
      return ("York over blurred segments points");
    else if (type == YORK_OVER_DSS)
      return ("York over digital straight segments");
    else if (type == YORK_WITH_CANNY)
      return ("York blend with Canny lines");
    else if (type == YORK_WITH_DSS)
      return ("York blend with digital straight lines");
    else return ("No info");
  }



private:

  /** York lines alone display modality. */
  static const int YORK_ALONE;
  /** Canny lines alone display modality. */
  static const int CANNY_ALONE;
  /** Dilated Canny lines alone display modality. */
  static const int DILATED_CANNY_ALONE;
  /** York over Canny display modality. */
  static const int YORK_OVER_CANNY;
  /** York over BS display modality. */
  static const int YORK_OVER_BS;
  /** York over DSS display modality. */
  static const int YORK_OVER_DSS;
  /** York with Canny display modality. */
  static const int YORK_WITH_CANNY;
  /** York with DSS display modality. */
  static const int YORK_WITH_DSS;
  /** No info display modality. */
  static const int NO_INFO;
  /** Default color for first drawn lines. */
  static const QColor UNDER_COLOR;
  /** Default color for further drawn lines. */
  static const QColor OVER_COLOR;
  /** Default color for both tools. */
  static const QColor BOTH_COLOR;
  /** Default value for pen width. */
  static const int DEFAULT_PEN_WIDTH;

  /** Set of York lines. */
  ExtLines *yorks;
  /** Set of Canny lines. */
  ExtLines *cannys;

  /** Background image. */
  const QImage *im;
  /** Grid width. */
  int w;
  /** Grid height. */
  int h;
  /** Segment detector. */
  BSDetector *det;
  /** Blurred segments map */
  bool *bsmap;
  /** York lines map */
  bool *clmap;
  /** Display modality */
  int type;
  /** Type of dilation for Canny lines */
  int cannyDilation;

  /** Displays blurred segments pixels */
  void drawBS (QPainter *painter);

  /** Displays blurred segments DSS */
  void drawDSS (QPainter *painter, bool disp = true);

  /** Displays a set of points */
  void drawPoints (QPainter *painter, vector<Pt2i> pts, bool disp = true);

  /** Displays York lines */
  void drawYorkLines (QPainter *painter);

  /** Displays Canny lines */
  void drawCannyLines (QPainter *painter, bool disp = true);

  /** Displays dilated Canny lines */
  void drawDilatedCannyLines (QPainter *painter, bool disp = true);

};

#endif
