#include <iostream>
#include <fstream>
#include <cmath>
#include "extlines.h"

using namespace std;


const int ExtLines::COV_DILATION = 4;


ExtLines::ExtLines (const char * name, int width, int height, int nbinfo)
{
  this->width = width;
  this->height = height;

  double val[nbinfo];
  int i = 0, nb = 0;
  ifstream input (name, ios::in);
  bool reading = true;
  if (input)
  {
    while (reading)
    {
      input >> val[i];
      if (input.eof ()) reading = false;
      if (++i == nbinfo)
      {
        ExtLine cl;
        cl.sx = val[0];
        cl.sy = height - 1 - val[1];
        cl.ex = val[2];
        cl.ey = height - 1 - val[3];
        exts.push_back (cl);
        i = 0;
        nb++;
      }
    }
  }
}


ExtLines::~ExtLines ()
{
}


double ExtLines::covering (const vector<BlurredSegment *> &segs) const
{
  bool cmap[width * height];
  for (int i = 0; i < width * height; i++) cmap[i] = false;
  vector<BlurredSegment *>::const_iterator it = segs.begin ();
  while (it != segs.end ())
  {
    DigitalStraightSegment *dss1 = (*it)->getSegment ();
    if (dss1 != NULL)
    {
      DigitalStraightSegment *dss = dss1->dilation (dss1->standard ());
      vector<Pt2i> points;
      dss->getPoints (points);
      vector<Pt2i>::iterator pit = points.begin ();
      while (pit != points.end ())
      {
        if (pit->x () >= 0 && pit->x () < width
            && pit->y () >= 0 && pit->y () < height)
          cmap[pit->x() + pit->y() * width] = true;
        pit++;
      }
      delete (dss);
    }
    it++;
  }

  int nbext = 0;
  int nbextin = 0;
  int nbextout = 0;
  int n = 0;
  Pt2i *pts = NULL;
  vector<ExtLine>::const_iterator yit = exts.begin ();
  while (yit != exts.end ())
  {
    n = 0;
    pts = Pt2i ((int) (yit->sx + 0.5), (int) (yit->sy + 0.5)).drawing (
          Pt2i ((int) (yit->ex + 0.5), (int) (yit->ey + 0.5)), &n);
    for (int i = 0; i < n; i++)
      if (cmap[pts[i].y () * width + pts[i].x ()]) nbextin ++;
      else nbextout ++;
    nbext += n;
    yit++;
    delete [] pts;
  }
  return (nbextin / (double) nbext);
}


double ExtLines::cannyLineCovering (const char *name,
                                    int &nb, double &lg, double minl) const
{
  // 1. Reading canny lines file
  vector<ExtLine> cannys;
  double val[5];
  double clLength = 0.;
  int i = 0;
  nb = 0;
  lg = 0.;
  ifstream input (name, ios::in);
  bool reading = true;
  if (input)
  {
    while (reading)
    {
      input >> val[i];
      if (input.eof ()) reading = false;
      if (++i == 5)
      {
        ExtLine cl;
        cl.sx = val[0];
        cl.sy = height - 1 - val[1];
        cl.ex = val[2];
        cl.ey = height - 1 - val[3];
        cannys.push_back (cl);
        clLength = sqrt ((cl.sx - cl.ex) * (cl.sx - cl.ex)
                         + (cl.sy - cl.ey) * (cl.sy - cl.ey));
        if (clLength > minl)
        {
          lg += clLength;
          nb++;
        }
        i = 0;
      }
    }
  }

  // 2. Spreading canny lines
  bool cmap[width * height];
  for (int i = 0; i < width * height; i++) cmap[i] = false;
  vector<ExtLine>::iterator it = cannys.begin ();
  while (it != cannys.end ())
  {
    int xs = (int) (it->sx + 0.5);
    int ys = (int) (it->sy + 0.5);
    int xe = (int) (it->ex + 0.5);
    int ye = (int) (it->ey + 0.5);
    int minx = (xe < xs ? xe : xs);
    int miny = (ye < ys ? ye : ys);
    int maxx = (xe < xs ? xs : xe);
    int maxy = (ye < ys ? ys : ye);
    DigitalStraightSegment *dss = new DigitalStraightSegment (
      Pt2i (xs, ys), Pt2i (xe, ye),
      DigitalStraightLine::DSL_NAIVE, minx, miny, maxx, maxy);
    dss->dilate (dss->standard ());
    vector<Pt2i> points;
    dss->getPoints (points);
    vector<Pt2i>::iterator pit = points.begin ();
    while (pit != points.end ())
    {
      if (pit->x () >= 0 && pit->x () < width
          && pit->y () >= 0 && pit->y () < height)
        cmap[pit->x() + pit->y() * width] = true;
      pit++;
    }
    delete (dss);
    it++;
  }

  // 3. Covering external lines
  int nbext = 0;
  int nbextin = 0;
  int nbextout = 0;
  int n = 0;
  Pt2i *pts = NULL;
  vector<ExtLine>::const_iterator yit = exts.begin ();
  while (yit != exts.end ())
  {
    n = 0;
    pts = Pt2i ((int) (yit->sx + 0.5), (int) (yit->sy + 0.5)).drawing (
          Pt2i ((int) (yit->ex + 0.5), (int) (yit->ey + 0.5)), &n);
    for (int i = 0; i < n; i++)
      if (cmap[pts[i].y () * width + pts[i].x ()]) nbextin ++;
      else nbextout ++;
    nbext += n;
    yit++;
    delete [] pts;
  }
  return (nbextin / (double) nbext);
}
