#ifndef EXT_LINES_H
#define EXT_LINES_H


#include <QString>
#include <vector>
#include "blurredsegment.h"

using namespace std;


/** 
 * @class ExtLines extlines.h
 * \brief Comparable lines set from an external tool.
 */
class ExtLines
{
public:

  /**
   * \brief Creates an external lines set.
   */
  ExtLines (const char *name, int width, int height, int nbinfos = 4);

  /**
   * \brief Deletes the external lines set.
   */
  ~ExtLines ();

  /**
   * \brief Returns the count of lines in the set.
   */
  inline int countOfLines () const { return ((int) exts.size ()); }

  /**
   * \brief Returns the start point X coordinate of a line.
   */
  inline int xStart (int index) const {
    return ((int) (exts[index].sx + 0.5)); }

  /**
   * \brief Returns the start point Y coordinate of a line.
   */
  inline int yStart (int index) const {
    return ((int) (exts[index].sy + 0.5)); }

  /**
   * \brief Returns the end point X coordinate of a line.
   */
  inline int xEnd (int index) const {
    return ((int) (exts[index].ex + 0.5)); }

  /**
   * \brief Returns the end point Y coordinate of a line.
   */
  inline int yEnd (int index) const {
    return ((int) (exts[index].ey + 0.5)); }

  /**
   * \brief Returns the covering ratio of given blurred segments.
   */
  double covering (const vector<BlurredSegment *> &segs) const;

  /**
   * \brief Returns the covering ratio of CannyLine segments.
   * @param name Output file name.
   * @param nb Extracted number of lines.
   * @param lb Extracted total length of lines.
   * @param minl Minimal length of extarcted lines.
   */
  double cannyLineCovering (const char *name,
                            int &nb, double &lg, double minl) const;



private:

  /** Dilation distance for lines covering estimation. */
  static const int COV_DILATION;

 /** Aggregation of segment extraction results with initial conditions. */
  struct ExtLine
  {
    /** External line start point X coordinate. */
    double sx;
    /** External line start point Y coordinate. */
    double sy;
    /** External line end point X coordinate. */
    double ex;
    /** External line end point Y coordinate. */
    double ey;
  };

  /** Set of recorded external lines. */
  vector<ExtLine> exts;

  /** Image width */
  int width;
  /** Image height */
  int height;

};

#endif
